@@ -1130,6 +1130,9 @@ setup_meta_tree_view (GtkTreeView *tree_view,
  					       "pixbuf", META_THEME_PIXBUF_COLUMN,
  					       NULL);
   renderer = gtk_cell_renderer_text_new ();
+
+  g_object_set (renderer, "ellipsize", PANGO_ELLIPSIZE_END, NULL);
+  
   gtk_tree_view_insert_column_with_attributes (tree_view,
  					       -1, NULL,
  					       renderer,
@@ -1148,14 +1151,6 @@ setup_meta_tree_view (GtkTreeView *tree_view,
 }
 
 
-static void
-gnome_meta_theme_installer_run_cb (GtkWidget *button,
-				   GtkWidget *parent_window)
-{
-  gnome_theme_installer_run (parent_window, NULL);
-}
-
-
 static void
 gnome_theme_save_clicked (GtkWidget *button,
 			  gpointer   data)
@@ -1274,7 +1269,7 @@ setup_dialog (GladeXML *dialog)
 
   gtk_widget_set_sensitive(WID("meta_theme_revert_button"),FALSE);
 
-  g_signal_connect (G_OBJECT (WID ("meta_theme_install_button")), "clicked", G_CALLBACK (gnome_meta_theme_installer_run_cb), parent);
+  g_signal_connect (G_OBJECT (WID ("meta_theme_install_button")), "clicked", G_CALLBACK (gnome_theme_installer_run_cb), parent);
 
   g_signal_connect (G_OBJECT (WID ("meta_theme_details_button")), "clicked", gnome_theme_details_show, NULL);
 
@@ -1517,7 +1512,7 @@ gnome_theme_manager_drag_data_received_cb (GtkWidget *widget, GdkDragContext *co
 		gnome_vfs_uri_list_unref (uris);
 	}
 
-	gnome_theme_installer_run (widget, filename);
+	gnome_theme_install_from_uri (filename);
 	g_free (filename);
 }
 
