@@ -37,7 +37,7 @@
 #define UNDER_TITLE_SPACING 0 /* manually insert 1 blank line of text */
 #define LINE_HEIGHT 1
 #define BORDERS 7
-#define MAX_ITEM_WIDTH	120
+#define PREFERRED_ITEM_WIDTH 120
 #define ITEMS_SEPARATION 12
 
 typedef struct _GnomeccCanvasPrivate GnomeccCanvasPrivate;
@@ -63,6 +63,8 @@ struct _GnomeccCanvasPrivate {
 
 	/* accessibility stuff */
 	GHashTable *accessible_children;
+
+	gboolean single_click;
 };
 
 typedef struct {
@@ -118,9 +120,6 @@ static void gnomecc_canvas_get_property (GObject      *object,
 					 GValue       *value,
 					 GParamSpec   *pspec);
 
-static void gnomecc_canvas_draw_background (GnomeCanvas *canvas, GdkDrawable *drawable,
-					    int x, int y, int width, int height);
-
 static void gnomecc_canvas_size_allocate   (GtkWidget     *canvas,
 					    GtkAllocation *allocation);
 static void gnomecc_canvas_style_set       (GtkWidget     *canvas,
@@ -137,13 +136,11 @@ static void
 gnomecc_canvas_class_init (GnomeccCanvasClass *class)
 {
 	GObjectClass     *object_class = G_OBJECT_CLASS (class);
-	GnomeCanvasClass *canvas_class = GNOME_CANVAS_CLASS (class);
 	GtkWidgetClass   *widget_class = GTK_WIDGET_CLASS (class);
 
 	object_class->set_property = gnomecc_canvas_set_property;
 	object_class->get_property = gnomecc_canvas_get_property;
 	object_class->finalize = gnomecc_canvas_finalize;
-	canvas_class->draw_background = gnomecc_canvas_draw_background;
 
 	widget_class->style_set = gnomecc_canvas_style_set;
 	widget_class->size_allocate = gnomecc_canvas_size_allocate;
@@ -192,29 +189,20 @@ gnomecc_canvas_init (GnomeccCanvas *canvas)
 	priv->rtl = (gtk_widget_get_direction (GTK_WIDGET (canvas)) == GTK_TEXT_DIR_RTL);
 
 	priv->accessible_children = g_hash_table_new (g_int_hash, g_int_equal);
+	priv->single_click = FALSE;
 
 	gtk_widget_show_all (GTK_WIDGET (canvas));
 }
 
 static gboolean
-single_click_activates (void)
+single_click_activates (GnomeccCanvas *canvas)
 {
-	static gboolean needs_init = TRUE;
-	static gboolean use_single_click = FALSE;
-	if (needs_init)  {
-		GConfClient *client = gconf_client_get_default ();
-		char *policy = gconf_client_get_string (client, "/apps/nautilus/preferences/click_policy", NULL);
-		g_object_unref (G_OBJECT (client));
+	GnomeccCanvasPrivate *priv;
+	
+	priv = GNOMECC_CANVAS_GET_PRIVATE (canvas);
 
-		if (policy != NULL) {
-			use_single_click = (0 == g_ascii_strcasecmp (policy, "single"));
-			g_free (policy);
-		}
-		needs_init = FALSE;
-	}
-
-	return	use_single_click;
-}
+	return priv->single_click;
+}	
 
 static void
 gnome_canvas_item_show_hide (GnomeCanvasItem *item, gboolean show)
@@ -228,7 +216,6 @@ gnome_canvas_item_show_hide (GnomeCanvasItem *item, gboolean show)
 static void
 setup_entry (GnomeccCanvas *canvas, ControlCenterEntry *entry)
 {
-	GnomeccCanvasPrivate *priv;
 	EntryInfo *ei;
 	GtkWidget *widget;
 	GtkStateType state;
@@ -236,7 +223,6 @@ setup_entry (GnomeccCanvas *canvas, ControlCenterEntry *entry)
 	if (!entry)
 		return;
 
-	priv = GNOMECC_CANVAS_GET_PRIVATE (canvas);
 	widget = GTK_WIDGET (canvas);
 	ei = entry->user_data;
 
@@ -334,22 +320,31 @@ cover_event (GnomeCanvasItem *item, GdkEvent *event, ControlCenterEntry *entry)
 {
 	EntryInfo *ei = entry->user_data;
 	GnomeccCanvas *canvas = ei->canvas;
+	GdkCursor *cursor;
 
 	switch (event->type) {
 	case GDK_ENTER_NOTIFY:
 		ei->highlighted = TRUE;
 		setup_entry (canvas, entry); /* highlight even if it is already selected */
+
+		if (single_click_activates (canvas)) {
+			cursor = gdk_cursor_new (GDK_HAND1);
+			gdk_window_set_cursor (GTK_WIDGET (canvas)->window, cursor);
+			gdk_cursor_unref (cursor);
+		}
+			
 		return TRUE;
 	case GDK_LEAVE_NOTIFY:
 		ei->highlighted = FALSE;
 		setup_entry (canvas, entry);
+		gdk_window_set_cursor (GTK_WIDGET (canvas)->window, NULL);
 		return TRUE;
 	case GDK_BUTTON_PRESS:
-		if (single_click_activates ()) {
+		select_entry (canvas, entry);
+
+		if (single_click_activates (canvas))
 			activate_entry (entry);
-		} else {
-			select_entry (canvas, entry);
-		}
+
 		return TRUE;
 	case GDK_2BUTTON_PRESS:
 		activate_entry (entry);
@@ -365,7 +360,6 @@ cb_canvas_event (GnomeCanvasItem *item, GdkEvent *event, GnomeccCanvas *canvas)
 	GnomeccCanvasPrivate *priv;
 	EntryInfo *ei = NULL;
 	gint n_category, n_entry;
-	gint n_categories, n_entries;
 
 	priv = GNOMECC_CANVAS_GET_PRIVATE (canvas);
 	
@@ -382,8 +376,6 @@ cb_canvas_event (GnomeCanvasItem *item, GdkEvent *event, GnomeccCanvas *canvas)
 
 	n_entry = 0;
 	n_category = 0;
-	n_categories = priv->info->n_categories;
-	n_entries = priv->info->categories[ei->n_category]->n_entries;
 
 	switch (event->key.keyval) {
 	case GDK_KP_Right:
@@ -446,7 +438,6 @@ cb_canvas_event (GnomeCanvasItem *item, GdkEvent *event, GnomeccCanvas *canvas)
 			activate_entry (priv->selected);
 
 		return TRUE;
-		break;
 	case GDK_Escape:
 		gtk_main_quit ();
 		return TRUE;
@@ -477,15 +468,14 @@ calculate_item_width (GnomeccCanvas *canvas, EntryInfo *ei)
 	priv   = GNOMECC_CANVAS_GET_PRIVATE (canvas);
 	layout = GNOME_CANVAS_TEXT (ei->text)->layout;
 
-	pango_layout_set_wrap (layout, PANGO_WRAP_WORD_CHAR);
+	pango_layout_set_wrap (layout, PANGO_WRAP_WORD);
 	pango_layout_set_width (layout, -1);
 	pango_layout_get_pixel_extents (layout, NULL, &rectangle);
 
 	/* If its too big wrap at the max and regen to find the layout */
-	if (rectangle.width > MAX_ITEM_WIDTH) {
-		pango_layout_set_width (layout, MAX_ITEM_WIDTH * PANGO_SCALE);
+	if (rectangle.width > PREFERRED_ITEM_WIDTH) {
+		pango_layout_set_width (layout, PREFERRED_ITEM_WIDTH * PANGO_SCALE);
 		pango_layout_get_pixel_extents (layout, NULL, &rectangle);
-		rectangle.width = MAX_ITEM_WIDTH;
 	}
 
 	ei->text_height = rectangle.height;
@@ -696,12 +686,11 @@ build_canvas (GnomeccCanvas *canvas)
 					gnome_canvas_pixbuf_get_type (),
 					"pixbuf", pixbuf,
 					NULL);
-				g_object_unref (pixbuf);
+
 				ei->highlight_pixbuf = gnome_canvas_item_new (ei->group,
 					gnome_canvas_pixbuf_get_type (),
 					"pixbuf", highlight_pixbuf,
 					NULL);
-				g_object_unref (highlight_pixbuf);
 			} else {
 				ei->pixbuf = NULL;
 				ei->highlight_pixbuf = NULL;
@@ -765,24 +754,16 @@ gnomecc_canvas_get_property (GObject      *object,
 static void
 gnomecc_canvas_finalize (GObject *object)
 {
+	GnomeccCanvasPrivate *priv;
+
+	priv = GNOMECC_CANVAS_GET_PRIVATE (object);
+
+	g_hash_table_destroy (priv->accessible_children);
+
 	if (G_OBJECT_CLASS (gnomecc_canvas_parent_class)->finalize)
 		(* G_OBJECT_CLASS (gnomecc_canvas_parent_class)->finalize) (object);
 }
 
-static void
-gnomecc_canvas_draw_background (GnomeCanvas *canvas, GdkDrawable *drawable,
-				int x, int y, int width, int height)
-{
-	/* By default, we use the style base color. */
-	gdk_gc_set_foreground (canvas->pixmap_gc,
-			       &GTK_WIDGET (canvas)->style->base[GTK_STATE_NORMAL]);
-	gdk_draw_rectangle (drawable,
-			    canvas->pixmap_gc,
-			    TRUE,
-			    0, 0,
-			    width, height);
-}
-
 /* A category canvas item contains all the elements
  * for the category, as well as the title and a separator
  */
@@ -973,6 +954,9 @@ set_style (GnomeccCanvas *canvas, gboolean font_changed)
 
 	priv = GNOMECC_CANVAS_GET_PRIVATE (canvas);
 
+	widget->style->bg[GTK_STATE_NORMAL] = widget->style->base[GTK_STATE_NORMAL];
+	control_center_reload_icons (priv->info);
+
 	for (i = 0; i < priv->info->n_categories; i++) {
 		CategoryInfo *catinfo = priv->info->categories[i]->user_data;
 
@@ -995,6 +979,15 @@ set_style (GnomeccCanvas *canvas, gboolean font_changed)
 		for (j = 0; j < priv->info->categories[i]->n_entries; j++) {
 			ControlCenterEntry *entry = priv->info->categories[i]->entries[j];
 			EntryInfo *entryinfo = entry->user_data;
+
+			g_object_set (entryinfo->pixbuf,
+				      "pixbuf", entry->icon_pixbuf,
+				      NULL);
+
+			g_object_set (entryinfo->highlight_pixbuf,
+				      "pixbuf", create_spotlight_pixbuf (entry->icon_pixbuf),
+				      NULL);
+
 			if (font_changed && entryinfo->text)
 				g_object_set (entryinfo->text,
 					      "font", NULL,
@@ -1240,12 +1233,8 @@ gnomecc_canvas_item_accessible_ref_state_set (AtkObject *object)
 static void
 gnomecc_canvas_item_accessible_class_init (AtkObjectClass *class)
 {
-	GObjectClass *object_class;
-
 	accessible_item_parent_class = g_type_class_peek_parent (class);
 
-	object_class = (GObjectClass *)class;
-
 	class->get_index_in_parent = gnomecc_canvas_item_accessible_get_index_in_parent; 
 	class->get_name = gnomecc_canvas_item_accessible_get_name;
 	class->get_parent = gnomecc_canvas_item_accessible_get_parent; 
@@ -1402,14 +1391,8 @@ gnomecc_canvas_accessible_initialize (AtkObject *accessible, gpointer data)
 static void
 gnomecc_canvas_accessible_class_init (AtkObjectClass *class)
 {
-	GObjectClass *object_class;
-	GtkAccessibleClass *accessible_class;
-
 	accessible_parent_class = g_type_class_peek_parent (class);
 
-	object_class = (GObjectClass *)class;
-	accessible_class = (GtkAccessibleClass *)class;
-
 	class->get_n_children = gnomecc_canvas_accessible_get_n_children;
 	class->ref_child = gnomecc_canvas_accessible_ref_child;
 	class->initialize = gnomecc_canvas_accessible_initialize;
@@ -1418,7 +1401,6 @@ gnomecc_canvas_accessible_class_init (AtkObjectClass *class)
 static gboolean
 gnomecc_canvas_accessible_add_selection (AtkSelection *selection, gint i)
 {
-	GnomeccCanvasPrivate *priv;
 	GnomeccCanvas *canvas;
 	GtkWidget *widget;
 	ControlCenterEntry *entry;
@@ -1429,7 +1411,6 @@ gnomecc_canvas_accessible_add_selection (AtkSelection *selection, gint i)
 		return FALSE;
 
 	canvas = GNOMECC_CANVAS (widget);
-	priv   = GNOMECC_CANVAS_GET_PRIVATE (canvas);
 
 	entry = gnomecc_canvas_accessible_get_entry (canvas, i);
 	select_entry (canvas, entry);
@@ -1722,3 +1703,15 @@ gnomecc_canvas_get_accessible (GtkWidget *widget)
 
 	return (* GTK_WIDGET_CLASS (gnomecc_canvas_parent_class)->get_accessible) (widget);
 }
+
+void
+gnomecc_canvas_set_single_click_mode (GnomeccCanvas *canvas, gboolean single_click)
+{
+	GnomeccCanvasPrivate *priv;
+
+	g_return_if_fail (GNOMECC_IS_CANVAS (canvas));
+
+	priv = GNOMECC_CANVAS_GET_PRIVATE (canvas);
+
+	priv->single_click = (single_click == TRUE);
+}
